import 'dart:io';
import 'dart:convert';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/directory_extensions.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/extensions/path_extension.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/models/freezed/build.yaml.dart';
import 'package:http/http.dart' as http;

String object = "";
String baseClassName = "BaseClass";

enum ModelType { map, list }

Future<void> initializeFreezed() async {
  await createFile(
    FileStructure(
      path: ".",
      name: "build",
      extension: "yaml",
      content: freezedBuildConfig,
    ),
  );

  String analysis = await readFileAsString("analysis_options.yaml");
  if (!analysis.contains("analyzer:")) {
    analysis = analysis.replaceAll("linter:", freezedAnalysis());
  } else if (!analysis.contains(freezedAnalysis())) {
    analysis = analysis.replaceBetweenTwoWords(begin: "analyzer:", end: "linter:", replace: freezedAnalysis());
  }

  await writeFileAsString("analysis_options.yaml", analysis);
}

Future<void> initModels() async {
  await initializeFreezed();
  // read json from file.
  stdout.writeln("Please Paste the path of your json file to convert it to Dart model\nExample: C:\\Users\\Public\\Downloads\\response.json");
  final path = stdin.readLineSync()?.trim();

  // get file name
  stdout.writeln("\nDart file name:");
  baseClassName = stdin.readLineSync() ?? "";
  if (baseClassName == "") {
    messageAndExit(message: "file name is not valid 😅");
  }

  if (path != null) {
    if (!path.endsWith(".json")) {
      messageAndExit(message: "file path should be end with .json 😅");
    }
    if (!path.asFile().existsSync()) {
      messageAndExit(message: "file path is not valid 😅");
    }
    stdout.writeln("\ngenerate model from specific key?, like `${terminalColor(label: "data", color: yellowColor)}`, if no press Enter.");
    final key = stdin.readLineSync();
    // convert file content to map
    final json = await readFileAsString(path);
    try {
      dynamic jsonModel = jsonDecode(json);
      if (jsonModel is Map<String, dynamic>) {
        // read from specific key
        if (key != null && key != "") {
          if (jsonModel[key] != null)
            jsonModel = jsonModel[key];
          else
            messageAndExit(message: "key content is not valid json 😅");
        }
        jsonModel = jsonModel as Map<String, dynamic>;
        object = JsonEncoder().convert(jsonModel);
        await json2dart(object);
      } else {
        messageAndExit(message: "file content is not valid json 😅");
      }
    } catch (error) {
      messageAndExit(message: error.toString());
    }
  } else {
    messageAndExit(message: "path is not valid 😅");
  }
}

Future<void> json2dart(dynamic object) async {
  stdout.writeln("\nConverting json to Dart...⌛");
  stdout.writeln("it may take a few mintues\n");
  // final localhost = "http://192.168.1.3:5000";
  final liveUrl = "https://develop.aghiadodeh.com";
  var url = Uri.parse('$liveUrl/json2dart/convert');

  var response = await http.post(url, body: {'json': object.toString()});
  if (response.statusCode == 200) {
    print("Creating dart files...\n");
    await splitDartClasses(response.body);
  } else {
    messageAndExit(message: "generate model faild 😅, be sure your json is valid\n${response.body}");
  }
}

Future<void> splitDartClasses(String classes) async {
  await Directory(modelsDataDir).createWithSubFolders();
  final splits = classes.split("class");
  final jsonTitle = "// ----------(json)----------";
  final jsonTitle2 = "// --------------------------";

  final exportFile = "${modelsDir}/data_models.dart".asFile();
  if (!exportFile.existsSync()) {
    exportFile.createSync();
  }

  String exports = await readFileAsString(exportFile.path);
  exports = exports.toString().trim();

  for (var i = 0; i < splits.length; i++) {
    final split = splits[i];
    if (split.trim().isNotEmpty) {
      String className = getClassName(split);
      String stringClass = "class${split}";
      String classContent = stringClass.replaceBetweenTwoWords(begin: "/*", end: "${className}({").replaceAll("${className}({", "");
      String jsonExample = stringClass.replaceBetweenTwoWords(begin: "/*", end: "*/");
      classContent = classContent.replaceAll(jsonExample, "").replaceAll(";", ",");
      if (className == "#Root#") className = baseClassName;
      String fileName = className.upperToUnderScore();
      bool isResposne = fileName.toLowerCase().contains("response");

      final export = "export '${isResposne ? 'responses' : 'data'}/${fileName}/${fileName}.dart';";
      if (!exports.contains(export)) exports = "$export\n$exports";

      final freezedClass = """
$widgetTitle
import ${freezedAnnotationPackage};
import '../../data_models.dart';
part '${fileName}.freezed.dart';
part '${fileName}.g.dart';

${jsonTitle}
${jsonExample}
${jsonTitle2}

@freezed
class ${className} with _\$${className} {
  const factory ${className}({
    ${classContent.trim()}
  }) = _${className};
  
  factory ${className}.fromJson(Map<String, dynamic> json) => _\$${className}FromJson(json);
}
""";
      // print(freezedClass);
      // print("fileName: ${fileName}");
      // print("className: ${className}");
      // print("jsonExample:\n${jsonExample}");
      // print("classContent:\n${classContent}");
      await createFile(
        FileStructure(
          path: "${isResposne ? modelsResponseDir : modelsDataDir}/${fileName}",
          name: fileName,
          content: () => freezedClass,
        ),
      );
    }
  }
  await writeFileAsString(exportFile.path, exports);
}

String getClassName(String string) {
  return string.split("\n").first.replaceAll(" ", "").replaceAll("{", "").trim();
}
