import 'dart:io';
import 'package:xflutter_cli/src/commands/i_flags.dart';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/extensions/nullable_extensions.dart';
import 'package:xflutter_cli/src/extensions/path_extension.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/models/formz_model.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/screen_widgets.dart';

class FormCreator {
  String basePath = "";
  String rootPath = "";
  String className = "";
  String paramsContent = "";
  String paramsImport = "";
  String viewModelContent = "";
  String viewModelImport = "";
  String widgetImport = "";
  String widgetContent = "";
  String paramsFile = "";
  String viewModelFile = "";
  List<FileStructure> files = [];
  String? type = "-";
  String? name;
  String field = "";
  List<String> types = [];
  List<String> names = [];

  Future<void> create(String path) async {
    if (!await Directory(singletonWidgetsDir).exists()) {
      messageAndExit(message: "please make sure you generate screen with same path and pass flag '--singleton'🤦‍♂️");
    }
    if (!await Directory(resourcesThemesDir).exists()) {
      messageAndExit(message: "please make sure you generate screen with same path and pass flag '--theme'🤦‍♂️");
    }
    basePath = path;
    rootPath = path.split("/").last;
    className = rootPath.capitalizeChar();
    checkRootPath();

    await initParams();
    await initViewModel();

    int i = 1;
    stdout.writeln(
      "Create your form as you wish 👀\njust enter the type of every textField you want to generate in the form\nWhen you finished please press Enter:",
    );
    while (type != null && type != "") {
      /// enter textField type
      stdout.writeln('\nPlease enter the type of textField, available types: ${FormType.values.map((e) => e.name).toList()}');
      type = stdin.readLineSync();
      if (type == null || type == "") break;
      if (!FormType.values.map((e) => e.name).contains(type)) {
        stdout.writeln("The type ($type) is not availabe❗\n---------------");
        continue;
      }

      /// enter textField name
      stdout.writeln("enter textField name:");
      name = stdin.readLineSync().defaultValue("formField${i}");
      name = name!.capitalizeChar();
      if (files.contains(name)) {
        stdout.writeln("this name already exists❗\n");
        continue;
      }
      field = "$name".firstLower();

      types.add(type!);
      names.add(name!);
      final formType = FormType.values.firstWhere((element) => element.name == type, orElse: () => FormType.text);
      final formzModel = formzMap[formType];
      if (formzModel != null) {
        addFiles(formzModel);
        appendParams(formzModel);
        appendViewModel(formzModel);
        appendWidget(formzModel);
      }
      type = confirmCommand(message: "${terminalColor(label: "\nadd new field?", color: yellowColor)}\n[yes/no]\n") ? "-" : null;
      i++;
    }

    if (i == 1) {
      messageAndExit(message: "you should enter one field at least");
    }

    stdout.writeln("\nform name: \ndefault is (${terminalColor(label: rootPath, color: yellowColor)}):\nmake sure the form name not contains `form` label");

    final formName = stdin.readLineSync().defaultValue("${rootPath}");

    await editParamsFile();
    await editViewModelFile();

    for (var i = 0; i < files.length; i++) {
      await createFile(files[i]);
    }

    await createFile(
      FileStructure(
        path: widgetsDir(path),
        name: "${formName.upperToUnderScore()}_form",
        content: () => widgetContnet(formName.capitalizeChar()),
      ),
    );
  }

  void checkRootPath() {
    final rootDir = viewmodelDir(basePath);
    if (!Directory(widgetsDir(basePath)).existsSync() ||
        !'${rootDir}/${rootPath}_viewmodel.dart'.asFile().existsSync() ||
        !'${rootDir}/${rootPath}_params.dart'.asFile().existsSync()) {
      messageAndExit(message: """
path is not valid❗\n
make sure the path contains these files:
- viewmodels/${rootPath}_viewmodel.dart
- viewmodels/${rootPath}_params.dart
- widgets/
""");
    }
  }

  void addFiles(FormzModel formzModel) {
    files.addAll([
      FileStructure(
        path: formsModelDir,
        name: formzModel.fileName,
        content: formzModel.fileContent,
      ),
    ]);
  }

  void appendParams(FormzModel formzModel) {
    paramsImport = paramsImport.appendIfNotContains("\nimport ${formzModel.import};");
    paramsImport = paramsImport.appendIfNotContains("\nimport ${liveDataPackage};");
    paramsContent = paramsContent.replaceAll("Params {", """
Params {
  /// ${className} ${field}
  ${formzModel.liveDataVariable(field)}""");
  }

  void appendViewModel(FormzModel formzModel) {
    if (type == FormType.confirm_password.name && !types.contains(FormType.password.name)) {
      messageAndExit(message: "cannot add confirm password without password❗");
    }
    viewModelImport = viewModelImport.appendIfNotContains("\nimport ${liveDataPackage};");
    viewModelImport = viewModelImport.appendIfNotContains("\nimport ${formzModel.import};");

    /// update value content
    String content = """
final newValue = ${formzModel.className}.dirty(value);
attr.postValue(newValue);""";
    if (type == FormType.confirm_password.name) {
      // confirm password required current value and password value.
      final index = types.indexWhere((element) => element == FormType.password.name);
      content = """
final password = params.${names[index].firstLower()}.value.value;
final newValue = ${formzModel.className}.dirty(password: password, value: value);
attr.postValue(newValue);""";
    }

    // add update callback method to viewModel
    if (!viewModelContent.contains(formzModel.className)) viewModelContent = viewModelContent.appendIfNotContains("""\n
/// update [${className}Params] some [${formzModel.className}] variable.
void ${formzModel.updateCallback}(MutableLiveData<${formzModel.className}> attr, String value) {
  ${content}
}""");
  }

  void appendWidget(FormzModel formzModel) {
    widgetImport = widgetImport.appendIfNotContains("import ${formzModel.import};\n");
    widgetContent = widgetContent.appendIfNotContains("""\n
    ///// ${className} ${name} /////
    ${formzModel.formField(field)}
    const SizedBox(height: formSpacing),
    """);
  }

  String widgetContnet(String fileName) {
    final theme = flags.contains(CliFlags.theme);

    final child = """
Column(
  mainAxisAlignment: MainAxisAlignment.spaceBetween,
  children: [
    Column(
      children: [
        $widgetContent
      ],
    ),

    ///// Form Submit /////
    LiveDataBuilder<bool>(
      data: viewModel.params.loading,
      builder: (context, boolean) {
        return SizedBox(
          width: double.infinity,
          child: CustomizedButton(
            enabled: !boolean,
            child: const Text("Submit"),
            callback: () {
              // TODO: Submit Form action
            },
          ),
        );
      },
    ),
  ],
)
""";

    return """
$widgetTitle
import ${materialPackage};
import ${liveDataPackage};
import 'package:${packageName}/ui/widgets/text_fields/livedata_text_field.dart';
import 'package:${packageName}/ui/widgets/buttons/customized_button.dart';
import 'package:${packageName}/ui/resources/dimensions/dimensions.dart';
${widgetImport}
import "package:${packageName}/ui/widgets/singleton/singleton_widget.dart";
import "../viewmodels/${rootPath}_viewmodel.dart";
${theme ? """
import 'package:${packageName}/ui/core/layouts/theme_widget.dart';
""" : ""}

class ${fileName}Form extends StatelessWidget {
  const ${fileName}Form({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SingletonWidget<${className}ViewModel>(
      builder: (viewModel) {
        return ${themeWrapper(theme, child: child)};
      },
    );
  }
}
""";
  }

  Future<void> initParams() async {
    final rootDir = viewmodelDir(basePath);
    paramsFile = await readFileAsString("$rootDir/${rootPath}_params.dart");
    paramsImport = paramsFile.replaceBetweenTwoWords(begin: "import ", end: "\nclass").replaceAll("\nclass", "");
    paramsContent = paramsFile.replaceBetweenTwoWords(begin: "Params {", end: "}").replaceAll("}", "");
  }

  Future<void> initViewModel() async {
    final rootDir = viewmodelDir(basePath);
    viewModelFile = await readFileAsString("$rootDir/${rootPath}_viewmodel.dart");
    viewModelImport = viewModelFile.replaceBetweenTwoWords(begin: "import ", end: "\nclass").replaceAll("\nclass", "");
    viewModelContent = viewModelFile.replaceBetweenTwoWords(begin: "ViewModel {", end: "}", lastIndex: true);
  }

  Future<void> editParamsFile() async {
    final rootDir = viewmodelDir(basePath);
    String params = paramsFile.replaceBetweenTwoWords(begin: "import ", end: "\nclass", replace: "$paramsImport\nclass");
    params = params.replaceBetweenTwoWords(begin: "Params {", end: "}", replace: paramsContent + "\n}");
    await writeFileAsString('${rootDir}/${rootPath}_params.dart', params);
  }

  Future<void> editViewModelFile() async {
    final rootDir = viewmodelDir(basePath);
    String viewModel = viewModelFile.replaceBetweenTwoWords(begin: "import ", end: "\nclass", replace: "$viewModelImport\nclass");
    viewModel = viewModel.replaceBetweenTwoWords(begin: "ViewModel {", end: "}", replace: viewModelContent + "\n", lastIndex: true);
    await writeFileAsString('${rootDir}/${rootPath}_viewmodel.dart', viewModel);
  }
}
