import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String customizedAnimatedWidget() => """
$widgetTitle
import ${materialPackage};

class CustomizedAnimatedWidget extends StatefulWidget {
  /// [child] is wrapped [Widget] inside [CustomizedAnimatedWidget].
  final Widget child;

  /// animation [Duration] default is Duration(milliseconds: 300).
  final Duration duration;

  /// make [AnimationController] starts after [Duration].
  final Duration delay;

  /// [manualTrigger] when set true you will disable animate [child] when view initialized
  final bool manualTrigger;

  /// animation sliding value.
  final double from;
  const CustomizedAnimatedWidget({
    Key? key,
    required this.child,
    this.duration = const Duration(milliseconds: 300),
    this.delay = const Duration(milliseconds: 100),
    this.manualTrigger = false,
    this.from = 50,
  }) : super(key: key);

  @override
  State<CustomizedAnimatedWidget> createState() => CustomizedAnimatedWidgetState();
}

class CustomizedAnimatedWidgetState extends State<CustomizedAnimatedWidget> with SingleTickerProviderStateMixin {
  AnimationController? controller;
  bool disposed = false;
  late Animation<double> animation;
  late Animation<double> opacity;
  @override
  void dispose() {
    disposed = true;
    controller!.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    controller = AnimationController(duration: widget.duration, vsync: this);
    animation = Tween<double>(begin: widget.from, end: 0).animate(CurvedAnimation(parent: controller!, curve: Curves.easeOut));
    opacity = Tween<double>(begin: 0, end: 1).animate(CurvedAnimation(parent: controller!, curve: const Interval(0, 1)));
    if (!widget.manualTrigger) {
      Future.delayed(widget.delay, () {
        if (!disposed) {
          controller?.forward();
        }
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: controller!,
      builder: (BuildContext context, Widget? child) {
        return Transform.translate(
          offset: Offset(0, animation.value),
          child: AnimatedOpacity(
            opacity: controller!.value,
            duration: widget.duration,
            child: widget.child,
          ),
        );
      },
    );
  }
}
""";

String scaleFadeWidget() => """
$widgetTitle
import ${materialPackage};

enum AnimationType { fade, scale, fadeScale }

class ScaleFadeWidget extends StatefulWidget {
  final Widget child;
  final Duration? duration;
  final AnimationType type;
  final double? fadeValue;
  final double? scaleValue;
  const ScaleFadeWidget({
    required this.child,
    this.duration,
    this.type = AnimationType.fadeScale,
    this.fadeValue,
    this.scaleValue,
    Key? key,
  }) : super(key: key);

  @override
  State<ScaleFadeWidget> createState() => ScaleFadeWidgetState();
}

class ScaleFadeWidgetState extends State<ScaleFadeWidget> with SingleTickerProviderStateMixin {
  double? _scale;
  AnimationController? _controller;
  Duration get _duration => widget.duration ?? const Duration(milliseconds: 150);
  @override
  void initState() {
    _controller = AnimationController(
      vsync: this,
      duration: _duration,
      lowerBound: 0.0,
      upperBound: widget.scaleValue ?? 0.025,
    )
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          _controller!.reverse();
        }
      })
      ..addListener(() {
        setState(() {});
      });
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
  }

  void animate({Function()? callback}) {
    _controller?.forward().then((value) => callback?.call());
  }

  @override
  Widget build(BuildContext context) {
    _scale = 1 - (_controller?.value ?? 0);
    return StatefulBuilder(
      builder: (context, setState) {
        switch (widget.type) {
          case AnimationType.scale:
            return Transform.scale(
              scale: _scale,
              child: widget.child,
            );

          case AnimationType.fade:
            return AnimatedOpacity(
              opacity: _controller?.value == 0 ? 1 : widget.fadeValue ?? 0.1,
              duration: _duration,
              child: widget.child,
            );

          case AnimationType.fadeScale:
          default:
            return Transform.scale(
              scale: _scale,
              child: AnimatedOpacity(
                opacity: _controller?.value == 0 ? 1 : widget.fadeValue ?? 0.35,
                duration: _duration,
                child: widget.child,
              ),
            );
        }
      },
    );
  }
}
""";

String animatedColumn() => """
$widgetTitle
import ${materialPackage};
import 'customized_animated_widget.dart';

class AnimatedColumn extends StatelessWidget {
  final List<Widget> children;
  final double animateDelay;
  final int duration;
  final MainAxisAlignment mainAxisAlignment;
  final CrossAxisAlignment crossAxisAlignment;
  final MainAxisSize mainAxisSize;

  /// [Column] with animate children one by one.
  ///
  /// [children] the list of widgets wrapped in Column.
  ///
  /// [duration] the animation duration in milliseconds.
  ///
  /// [mainAxisAlignment] the Column [MainAxisAlignment].
  ///
  /// [crossAxisAlignment] the Column [CrossAxisAlignment].
  ///
  /// [mainAxisSize] the Column [MainAxisSize].
  const AnimatedColumn({
    required this.children,
    this.animateDelay = 20, // delay between every view and previousView
    this.duration = 350,
    this.mainAxisAlignment = MainAxisAlignment.start,
    this.crossAxisAlignment = CrossAxisAlignment.center,
    this.mainAxisSize = MainAxisSize.max,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    _duration(Widget e) => Duration(milliseconds: duration + (children.indexOf(e) * animateDelay).toInt());

    return Column(
      mainAxisAlignment: mainAxisAlignment,
      crossAxisAlignment: crossAxisAlignment,
      mainAxisSize: mainAxisSize,
      children: children.map((e) => CustomizedAnimatedWidget(duration: _duration(e), child: e)).toList(),
    );
  }
}
""";

String animatedGesture() => """
$widgetTitle
import ${materialPackage};
import 'scale_fade_widget.dart';

class AnimatedGesture extends StatefulWidget {
  final Widget child;
  final Function() callback;
  final double? fadeValue;
  final double? scaleValue;
  final AnimationType? animationType;
  const AnimatedGesture({
    required this.child,
    required this.callback,
    this.fadeValue,
    this.scaleValue,
    this.animationType,
    Key? key,
  }) : super(key: key);

  @override
  State<AnimatedGesture> createState() => _AnimatedGestureState();
}

class _AnimatedGestureState extends State<AnimatedGesture> {
  final GlobalKey<ScaleFadeWidgetState> globalKey = GlobalKey<ScaleFadeWidgetState>();
  @override
  Widget build(BuildContext context) {
    return ScaleFadeWidget(
      key: globalKey,
      fadeValue: widget.fadeValue,
      scaleValue: widget.scaleValue,
      type: widget.animationType ?? AnimationType.fadeScale,
      child: InkWell(
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
        focusColor: Colors.transparent,
        hoverColor: Colors.transparent,
        onTap: () async {
          globalKey.currentState?.animate(callback: widget.callback);
        },
        child: widget.child,
      ),
    );
  }
}
""";

String animatedColorWidget() => """
$widgetTitle
import ${materialPackage};

class AnimatedColorWidget extends StatefulWidget {
  final Color fromColor;
  final Color toColor;
  final Duration duration;
  final Widget Function(AnimationController, Animation<dynamic>) builder;

  /// Change color of your widget with animation.
  /// 
  /// Just call controller.forward() or controller.reverse() inside [builder]
  /// 
  /// example: 
  // AnimatedColorWidget(
  //   fromColor: Colors.blue,
  //   toColor: Colors.green,
  //   builder: (controller, animation) {
  //     {condition} ? controller.forward() : controller.reverse();
  //     return Container(color: animation.value, width: 50, height: 50);
  //   },
  // )
  const AnimatedColorWidget({
    required this.fromColor,
    required this.toColor,
    this.duration = const Duration(milliseconds: 300),
    required this.builder,
    Key? key,
  }) : super(key: key);

  @override
  State<AnimatedColorWidget> createState() => AnimatedColorWidgetState();
}

class AnimatedColorWidgetState extends State<AnimatedColorWidget> with SingleTickerProviderStateMixin {
  late AnimationController animationController;
  late Animation colorTween;

  @override
  void initState() {
    animationController = AnimationController(vsync: this, duration: widget.duration);
    colorTween = ColorTween(begin: widget.fromColor, end: widget.toColor).animate(animationController);
    super.initState();
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: colorTween,
      builder: (context, child) {
        return widget.builder.call(animationController, colorTween);
      },
    );
  }
}
""";
